﻿using System.Collections.ObjectModel;
using System.Windows.Input;
using Microsoft.Maui.Controls;
using RobobinApp.Views;
using Plugin.BLE;
using Plugin.BLE.Abstractions.Contracts;
using Plugin.BLE.Abstractions.Exceptions;

namespace RobobinApp.ViewModels
{
    public class ConnectionPageViewModel : BaseViewModel
    {
        private BluetoothDevice _selectedDevice;
        public ICommand GoHomeCommand { get; }
        public ObservableCollection<BluetoothDevice> BluetoothDevices { get; }
        public ICommand ConnectCommand { get; }

        // BLE plugin fields
        private readonly IAdapter _adapter;
        private readonly IBluetoothLE _bluetoothLE;

        public ConnectionPageViewModel()
        {
            BluetoothDevices = new ObservableCollection<BluetoothDevice>();
            ConnectCommand = new Command(OnConnect);
            GoHomeCommand = new Command(async () => await OnGoHome());

            // Initialize BLE manager and adapter
            _bluetoothLE = CrossBluetoothLE.Current;
            _adapter = _bluetoothLE.Adapter;

            ScanDevices(); // Start scanning for devices
        }

        public BluetoothDevice SelectedDevice
        {
            get => _selectedDevice;
            set
            {
                _selectedDevice = value;
                OnPropertyChanged();
            }
        }

        public async Task OnGoHome()
        {
            await Application.Current.MainPage.Navigation.PushAsync(new MainPage());
        }

        private async void ScanDevices()
        {
            // Clear the current list of devices before scanning
            BluetoothDevices.Clear();

            try
            {
                if (!_bluetoothLE.IsAvailable)
                {
                    await Application.Current.MainPage.DisplayAlert("Bluetooth", "Bluetooth is not available.", "OK");
                    return;
                }

                if (!_bluetoothLE.IsOn)
                {
                    await Application.Current.MainPage.DisplayAlert("Bluetooth", "Please turn on Bluetooth.", "OK");
                    return;
                }

                _adapter.DeviceDiscovered += (s, a) =>
                {
                    // Add discovered devices to the list
                    BluetoothDevices.Add(new BluetoothDevice
                    {
                        Name = a.Device.Name ?? "Unknown Device",
                        MacAddress = a.Device.Id.ToString()
                    });
                };

                // Start scanning for devices
                await _adapter.StartScanningForDevicesAsync();
            }
            catch (Exception ex)
            {
                await Application.Current.MainPage.DisplayAlert("Error", $"Failed to scan for devices: {ex.Message}", "OK");
            }
        }

        private async void OnConnect()
        {
            if (SelectedDevice == null) return;

            // Logic to connect to the selected Bluetooth device
            await Application.Current.MainPage.DisplayAlert("Connection", $"Connecting to {SelectedDevice.Name}...", "OK");

            try
            {
                // Stop scanning before attempting connection
                await _adapter.StopScanningForDevicesAsync();

                // Find the device using the MAC address
                var device = await _adapter.ConnectToKnownDeviceAsync(Guid.Parse(SelectedDevice.MacAddress));

                // Display successful connection
                await Application.Current.MainPage.DisplayAlert("Connected", $"Connected to {SelectedDevice.Name}.", "OK");
            }
            catch (DeviceConnectionException ex)
            {
                // Handle connection failure
                await Application.Current.MainPage.DisplayAlert("Connection Error", $"Failed to connect: {ex.Message}", "OK");
            }
        }
    }

    public class BluetoothDevice
    {
        public string Name { get; set; }
        public string MacAddress { get; set; }
    }
}
